import hypermedia.video.*;
import peasy.*;

OpenCV opencv;
PeasyCam cam;
imageApplet s;

boolean autoRotate = true;
boolean invert = false;
boolean blur = false;
boolean contrast = false;
boolean bright = false;

void setup() {
  size(600,600,P3D);

  PFont myFont = createFont(PFont.list()[0], 64);
  textFont(myFont);
  textSize(18);

  opencv = new OpenCV();
  opencv.capture(400,300);
  new imageFrame(opencv);

  cam = new PeasyCam(this, 500);
  cam.setMinimumDistance(200);
  cam.setMaximumDistance(700);
  cam.rotateX(PI);
  cam.rotateY(.5);
  cam.setState(cam.getState());
  
  requestFocus();
}

void keyPressed() {
  if(key==' ')
    autoRotate = !autoRotate;
  else if(key=='i')
    invert = !invert;
  else if(key=='b')
    blur = !blur;
  else if(key=='c')
    contrast = !contrast;
  else if(key=='v')
    bright = !bright;
}

void draw() {
  // Inscrustation video
  opencv.read();  
  if(invert)
    opencv.invert();
  if(blur)
    opencv.blur(OpenCV.BLUR, 10);
  if(contrast)
    opencv.contrast(32);
  if(bright)
    opencv.brightness(32);
  s.image(opencv.image(),0,0);
  s.redraw();

  // 3D Histogram
  if(autoRotate)
    cam.rotateY(.01);    
  background(255);
  smooth(); noFill();
  stroke(128); box(256);

  // Points
  translate(-128,-128,-128);
  noSmooth();
  drawAxes();
  drawPoints(opencv.image());
}

// draw 3D RGB histogram with Points
void drawPoints(PImage img) {
  for(color c : img.pixels) {
    stroke(c);
    point(red(c), green(c), blue(c));
  }
}

// draw 3D RGB histogram with Boxes
void drawBoxes(PImage img) {
  for(color c : img.pixels) {
    noStroke();
    fill(c);
    pushMatrix();
    translate(red(c),green(c),blue(c));
    box(1);
    popMatrix();
  }
}

// draw 3D RGB histogram with Lines
void drawLines(PImage img) {
  color c_ = color(0);
  for(color c : img.pixels) {
    stroke(c);
    line(red(c),green(c),blue(c),
         red(c_),green(c_),blue(c_));
    c_ = c;
  }  
}

// draw 3D RGB histogram color axes
void drawAxes() {
  float[] rot = cam.getRotations();

  pushMatrix();
  translate(-10,-10,-10);
  rotateX(rot[0]);
  rotateY(rot[1]);
  rotateZ(rot[2]);
  fill(0);
  textAlign(CENTER);
  text("O",0,0,0);
  popMatrix();

  pushMatrix();
  translate(265,-10,-10);
  rotateX(rot[0]);
  rotateY(rot[1]);
  rotateZ(rot[2]);
  fill(255,0,0);
  textAlign(CENTER);
  text("R",0,0,0);
  popMatrix();

  pushMatrix();
  translate(-10,265,-10);
  rotateX(rot[0]);
  rotateY(rot[1]);
  rotateZ(rot[2]);
  fill(0,255,0);
  textAlign(CENTER);
  text("G",0,0,0);
  popMatrix();

  pushMatrix();
  translate(-10,-10,265);
  rotateX(rot[0]);
  rotateY(rot[1]);
  rotateZ(rot[2]);
  fill(0,0,255);
  textAlign(CENTER);
  text("B",0,0,0);
  popMatrix();
}

public class imageFrame extends Frame {
    public imageFrame(OpenCV opencv) {
        PImage img = opencv.image();
        setBounds(screen.width/2,100,img.width,img.height);
        s = new imageApplet(opencv);
        add(s);
        s.init();
        show();
        setResizable(false);
    }
}

public class imageApplet extends PApplet {

    PImage img;

    public imageApplet(OpenCV opencv) {
      img = opencv.image();
    }
    
    public void setup() {
        size(img.width, img.height);
        noLoop();
    }

    public void draw() {
    }
}
